@@ -103,14 +103,16 @@
     if (is.null(cenv)) {
         cenv <- new.env()
         assign("depth", 0L, envir = cenv)
+        assign("splitvars", rep(0L, length(partyvars)), envir = cenv)
     }
     depth <- get("depth", envir = cenv)
     assign("maxid", id, envir = cenv)
     if (depth >= ctrl$maxdepth)
         return(partynode(as.integer(id)))
 
+
     ### check for stumps
-    if (id > 1L && ctrl$stump) 
+    if (id > 1L && ctrl$stump)
         return(partynode(as.integer(id)))
 
     ### sw is basically the number of observations
@@ -128,10 +130,19 @@
     if (sw < ctrl$minsplit) 
         return(partynode(as.integer(id)))
 
-    svars <- which(partyvars > 0)
+    ### split variables used so far
+    splitvars <- get("splitvars", envir = cenv)
+    if (sum(splitvars) < ctrl$maxvar) {
+        ### all variables subject to splitting
+        svars <- which(partyvars > 0)
+    } else {
+        ### only those already used for splitting in other nodes are
+        ### eligible
+        svars <- which(partyvars > 0 & splitvars > 0)
+    }
     if (ctrl$mtry < Inf) {
-        mtry <- min(sum(partyvars > 0), ctrl$mtry)
-        svars <- .resample(svars, mtry, prob = partyvars[partyvars > 0])
+        mtry <- min(length(svars), ctrl$mtry)
+        svars <- .resample(svars, mtry, prob = partyvars[svars])
     } 
 
     thismodel <- trafo(subset = subset, weights = weights, info = info, 
@@ -225,6 +236,8 @@
     ret <- partynode(as.integer(id))
     ret$split <- thissplit
     ret$info <- info
+    splitvars[thissplit$varid] <- 1L
+    assign("splitvars", splitvars, cenv)
 
     ### determine observations for splitting (only non-missings)
     snotNA <- subset[!subset %in% data[[varid_split(thissplit), type = "missings"]]]
@@ -559,9 +572,9 @@ extree_data <- function(formula, data, subset, na.action = na.pass, weights, off
 
     ## extract variable lists
     vars <- list(
-      y = attr(mt$y, "term.labels"),
-      x = unique(unlist(lapply(grep("^x", names(mt)), function(i) attr(mt[[i]], "term.labels")))),
-      z = attr(mt$z, "term.labels"),
+      y = .get_term_labels(mt$y),
+      x = unique(unlist(lapply(grep("^x", names(mt)), function(i) .get_term_labels(mt[[i]])))),
+      z = .get_term_labels(mt$z),
       weights = if("(weights)" %in% names(mf)) "(weights)" else NULL,
       offset  = if("(offset)"  %in% names(mf)) "(offset)"  else NULL,
       cluster = if("(cluster)" %in% names(mf)) "(cluster)" else NULL,
@@ -675,11 +688,15 @@ extree_data <- function(formula, data, subset, na.action = na.pass, weights, off
     } else { ### ytype = "matrix"
         Ytmp <- model.matrix(~ 0 + ., Formula::model.part(formula, yxmf, lhs = TRUE))
         ### <FIXME> are there cases where Ytmp already has missings? </FIXME>
-        if (length(ret$yxmissings) == 0) {
+        if (is.finite(nmax["yx"])) {
             Ymat <- Ytmp
         } else {
-            Ymat <- matrix(0, nrow = NROW(yxmf), ncol = NCOL(Ytmp))
-            Ymat[-ret$yxmissings,] <- Ytmp
+            if (length(ret$yxmissings) == 0) {
+                Ymat <- Ytmp
+            } else {
+                Ymat <- matrix(0, nrow = NROW(yxmf), ncol = NCOL(Ytmp))
+                Ymat[-ret$yxmissings,] <- Ytmp
+            }
         }
         yx <- list("y" = Ymat)
     }
@@ -687,11 +704,15 @@ extree_data <- function(formula, data, subset, na.action = na.pass, weights, off
       ni <- paste("x", if(i == 1L) "" else i, sep = "")
       ti <- if(!ymult & npart == 2L) mt$yx else mt[[ni]]
       Xtmp <- model.matrix(ti, yxmf)
-      if (length(ret$yxmissings) == 0) {
+      if (is.finite(nmax["yx"])) {
           Xmat <- Xtmp
       } else {
-          Xmat <- matrix(0, nrow = NROW(yxmf), ncol = NCOL(Xtmp))
-          Xmat[-ret$yxmissings,] <- Xtmp
+          if (length(ret$yxmissings) == 0) {
+              Xmat <- Xtmp
+          } else {
+              Xmat <- matrix(0, nrow = NROW(yxmf), ncol = NCOL(Xtmp))
+              Xmat[-ret$yxmissings,] <- Xtmp
+          }
       }
       yx[[ni]] <- Xmat
       if(ncol(yx[[ni]]) < 1L) {
@@ -718,6 +739,25 @@ model.frame.extree_data <- function(formula, yxonly = FALSE, ...) {
     return(formula$data[, c(vars$y, vars$x, vars$offset, vars$cluster),drop = FALSE])
 }    
 
+## for handling of non-standard variable names within extree_data() by mimicking 
+## handling of such variables in model.frame() etc. in "stats" prompted by
+## https://stackoverflow.com/questions/64660889/ctree-ignores-variables-with-non-syntactic-names
+
+.deparse_variables <- function(x) paste(deparse(x, width.cutoff = 500L,
+  backtick = !is.symbol(x) && is.language(x)), collapse = " ")
+
+.get_term_labels <- function(terms, delete_response = FALSE) {
+  ## ## with just standard variable names one could use:
+  ## attr(terms, "term.labels")
+
+  ## delete response from terms (if needed)
+  if(delete_response) terms <- delete.response(terms)
+
+  ## with non-standard variable names -> deparse to handle `...` correctly
+  vapply(attr(terms, "variables"), .deparse_variables, " ")[-1L]
+}
+
+
 ### <FIXME> document how to extract slots fast </FIXME>
 "[[.extree_data" <- function(x, i, type = c("original", "index", "scores", "missings")) {
     type <- match.arg(type, choices = c("original", "index", "scores", "missings"))
@@ -760,6 +800,7 @@ extree_control <- function
     minbucket = 7L, 
     minprob = 0.01, 
     nmax = Inf,
+    maxvar = Inf,
     stump = FALSE,
     lookahead = FALSE, ### try trafo() for daugther nodes before implementing the split
     maxsurrogate = 0L, 
@@ -797,7 +838,7 @@ extree_control <- function
 
     list(criterion = criterion, logmincriterion = logmincriterion,
          minsplit = minsplit, minbucket = minbucket, 
-         minprob = minprob, stump = stump, nmax = nmax,
+         minprob = minprob, maxvar = max(c(1, maxvar)), stump = stump, nmax = nmax,
          lookahead = lookahead, mtry = mtry,
          maxdepth = maxdepth, multiway = multiway, splittry = splittry,
          maxsurrogate = maxsurrogate, 
